#!/usr/bin/env python3

import os
import sys
import time
import intelhex
import subprocess
import serial.tools.list_ports
from test_common import (
    DeviceUnderTestAeadUARTP,
    eprint,
    test_main,
    FileMutex
)


def get_serial():
    ports = serial.tools.list_ports.comports()
    devices = [
        p.device
        for p in ports
        if (p.vid == 0x1A86 and p.pid == 0x7523)
    ]
    devices.sort()
    return devices[0]


class Uno(DeviceUnderTestAeadUARTP):

    def __init__(self, build_dir):
        DeviceUnderTestAeadUARTP.__init__(self)

        self.uart_device = get_serial()
        devname = os.path.basename(self.uart_device)
        self.lock = FileMutex('/var/lock/lwc-compare.%s.lock' % devname)
        self.build_dir = build_dir
        self.template_path = os.path.dirname(sys.argv[0])

        self.firmware_path = os.path.join(
            build_dir, 'firmware.hex')

        ih = intelhex.IntelHex()
        ih.loadhex(self.firmware_path)
        total_size = 0
        for start, stop in ih.segments():
            print(start, stop, stop-start)
            total_size += stop-start
        self._firmware_size = total_size

    def firmware_size(self):
        return self._firmware_size

    def reset(self):
        self.ser.setDTR(True)
        time.sleep(0.01)
        self.ser.setDTR(False)
        time.sleep(1)

    def flash(self):
        pipe = subprocess.PIPE
        avrdude_dir = '/home/tester/.platformio/packages/tool-avrdude/'

        cmd = [os.path.join(avrdude_dir, 'avrdude')]
        cmd += ['-C', os.path.join(avrdude_dir, 'avrdude.conf')]
        cmd += ['-v']
        cmd += ['-p', 'atmega328p']
        cmd += ['-c', 'arduino']
        cmd += ['-P', self.uart_device]
        cmd += ['-b', '115200']
        cmd += ['-D']
        cmd += ['-U', 'flash:w:%s:i' % os.path.abspath(self.firmware_path)]

        self.ser = serial.Serial(
            self.uart_device,
            baudrate=115200,
            timeout=5)
        self.ser.setDTR(True)
        time.sleep(0.01)
        self.ser.setDTR(False)
        self.ser.close()

        p = subprocess.Popen(
            cmd, stdout=sys.stderr, stdin=pipe)
        stdout, stderr = p.communicate("")
        assert p.returncode == 0
        eprint("Firmware flashed.")

        self.ser = serial.Serial(
            self.uart_device,
            baudrate=115200,
            timeout=5)
        self.reset()

    def dump_ram(self):
        return None


if __name__ == "__main__":
    sys.exit(test_main(Uno, 0x0800, sys.argv))
